use std::io::{self, Write};
use num_bigint::BigUint;

fn log_2(x: BigUint) -> u64 {
    x.bits() - 1
}

fn is_prime(number: BigUint) -> bool {
    if BigUint::from(1u8) == number {
        return false;
    }
    if BigUint::from(4u8) > number  {
        return true;
    }

    let two = BigUint::from(2u8);

    // number = 2^a - 1
    // a = log2(number + 1)
    let a = log_2(number.clone()+1u8);
    if BigUint::from(2u8).pow(a as u32)-BigUint::from(1u8) != number {
        //eprintln!("only supports numbers of type (2^x)-1");
        let mut i = BigUint::from(0u8);
        let one = BigUint::from(1u8);
        let zero = BigUint::from(0u8);
        
        let sqrtnum = number.sqrt()+one.clone();
        let mut is_prime = true;
        loop {
            i = i + one.clone();
            if i == sqrtnum {
                break;
            }
            if sqrtnum.clone()%i.clone() == zero {
                is_prime = false;
                break;
            }
        }
        return is_prime;
    }

    // 4 12 194
    let mut last = BigUint::from(4u8);
    
    for _i in 2..a {
        last = (last.pow(2)-two.clone())%number.clone();
    }
    if last == BigUint::from(0u8) {
        true
    } else {
        false
    }
}

fn main() {
    let mut buffer = String::new();
    let stdin = io::stdin();
    print!("please enter the number to check: ");
    io::stdout().flush().expect("could not flush");
    stdin.read_line(&mut buffer).expect("could not read from stdin");
    buffer = buffer.trim().to_owned();
    
    let number = BigUint::parse_bytes(&buffer.as_bytes(), 10).expect("expected positive number");

    if is_prime(number.clone()) {
        println!("\n{number} is a prime");
    } else {
        println!("\n{number} is not a prime");
    }
}
